/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Mobile Browser Simulator
 * (c) Copyright IBM Corporation 2011, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

/**
 * Widget representing a single mobile browser simulator
 */
define("ibm_mobile/MobileBrowserSimulator",[
	"dojo",
	"dojo/dom-style",
 	"dijit",
 	"dijit/_Widget",
 	"dijit/_TemplatedMixin",
 	"dijit/_WidgetsInTemplateMixin",
 	"dojo/text!ibm_mobile/templates/MobileBrowserSimulator.html",
 	"dijit/form/Button",
 	"dijit/Tooltip",
 	"dojo/i18n",
 	"dojo/window",
 	"dojox/css3/fx",
 	"ibm_mobile/MobileDeviceTree",
    "dijit/TooltipDialog",
    "dijit/form/TextBox",
    "dijit/form/Button",
    "dijit/form/DropDownButton",
    "dijit/DropDownMenu",
    "dijit/MenuItem",
    "dojox/form/DropDownSelect",
    "dojo/data/ObjectStore",
    "dojo/store/Memory",
    "dijit/Dialog",
    "dijit/form/Select",
    "dojo/mouse",
    "dojo/on",
    "dijit/layout/ContentPane",
    "dojo/query",
    "dojo/_base/connect",
    "dijit/popup",
    "dijit/place",
    "ibm_mobile/QRCodeDialog",
    "dijit/Menu",
    "dijit/form/RadioButton",
    "dojo/query",
    "dijit/form/Form",
    "dojo/dom-attr",
], function(dojo, domStyle, dijit, _Widget, _TemplatedMixin, _WidgetsInTemplateMixin, mobileBrowserSimulatorTemplate,TooltipDialog, TextBox, Button, DropDownButton,DropDownMenu, MenuItem, DropDownSelect, ObjectStore, Memory,Dialog,Select,mouse,on,ContentPane,query,connect,popup,place,QRCodeDialog,Menu,RadioButton,query,Form,domAttr) {

	var MobileBrowserSimulator = dojo.declare("ibm_mobile.MobileBrowserSimulator",
			[dijit._Widget, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin ], {

		/** Used to debug this widget */
		__DEBUG_ENABLED : false,

		/** Used to check if the same domain is used for setting the src url**/
		sameDomain : true,

		// PRIVATE CONST
		/** When hacks fail to calculate the scroll bar width, use this average scroll bar width */
		DEFAULT_SCROLL_BAR_WIDTH : 17,

		/**
		 * Default viewport device width to use if a device does not specify one.
		 */
		DEFAULT_VIEWPORT_DEVICE_WIDTH : 320,

		/** the default page to load in the simulator if no page is specified */
		DEFAULT_PAGE : dojo.moduleUrl("ibm_mobile", "blank_viewport.html"),

		/** skins for the known platforms */
		KNOWN_PLATFORM_SKINS : {
			'platform.blackberry' : 'bb',
			'platform.android' : 'android',
			'platform.ios.ipad' : 'ios',
			'platform.ios.iphone' : 'ios',
			'platform.windowsphone' : 'windowsphone',
			'platform.windows8' : 'windows8'
		},

		/** default skin to use if platform is unknown */
		DEFAULT_SKIN : 'android',

		// PUBLIC PARAMS
		/** IFRAME (DOJO ATTACH POINT) */
		iFrame : null,
		device : null,
		platform : null,
		uuid : null,
		isRotated : false,
		/** @type dojo.data.ItemFileReadStore */
		deviceStore : null,
		/** @type dojo.data.ItemFileReadStore */
		skinsStore : null,
		/** @type dojo.data.ItemFileReadStore */
		environmentsStore : null,
		scaleParams : null,
		/** @type String */
		webpage : null,
		/** @type String */
		smartPreview : null,

		iosVersion: null,

		iosVersionOn : null,

		// PUBLIC EVENTS
		/**
		 * <p>Event invoked when this simulator is closed.</p>
		 *
		 * @param self the simulator that was closed
		 */
		onClose : function(self) {
			dojo.publish('/mbs/deviceremoved', [this.uuid]);
		},

		/**
		 * <p>Event invoked when this simulator has finished rotating.</p>
		 */
		onRotate : function() {
			this.publishDeviceChange();
		},

		/**
		 * <p>Event invoked when this simulator has changed device.</p>
		 */
		onDeviceChange : function(device){
			this.publishDeviceChange();
		},

		// WIDGET PARAMS
		templateString : mobileBrowserSimulatorTemplate,
		widgetsInTemplate : true,

		// PRIVATE
		_horzScrollBarSize : 0,
		_vertScrollBarSize : 0,
		_horzScrollbarVisible : false,
		_vertScrollbarVisible : false,
		_subscriptions : null,
		_frameWidth : 0,
		_frameHeight : 0,
		_prevScaledRatio : 1,
		_prevSkinClass: null,
		_fitToWindowConnections : null,
		_forceNoScrollBar : true,
		_isChangingDevice : false,
		_isRotating : false,
		_updateCallsCount: 0,
		_watchBodySizeTimer: null,
		_radioDiv : null,

		_viewport : null,

		_layout : null,

		/**
		 * <p>the body width defined by the loaded page</p>
		 *
		 * @type Number
		 */
		_bodyTagDefinedWidth : null,

		/**
		 * <p>the body height defined by the loaded page</p>
		 *
		 * @type Number
		 */
		_bodyTagDefinedHeight : null,

		// DOJO ATTACH POINTS
		/** DIV */
		_wrapper : null,
		/** @type dijit.form.Button */
		_closeBtn : null,
		/** @type dijit.form.DropDownButton */
		_deviceBtn : null,
		/** @type dijit.form.Button */
		_qrcodeButtonDevice : null,
		/** @type dijit.form.DropDownButton */
		urlDeviceGoButton: null,
		/** @type dijit.form.Form */
		myForm1: null,
		/** DIV */
		_warning : null,
		/** IMG */
		_spinner : null,

		/**
		 * <p>Widget lifecycle function.</p>
		 *
		 * @param params
		 */
		constructor : function(params) {
			this.__log('constructor', arguments);

			//init fields
			this.webpage = this.DEFAULT_PAGE;
			this._subscriptions = [];
			this._fitToWindowConnections = [];

			this._viewport = {
				width : 0,
				height: 0,
				scale: 1
			};

			this._layout = {
				width : 0,
				height : 0
			};
		},

		/**
		 * <p>Publish device change</p>
		 */
		publishDeviceChange : function() {
			 dojo.publish('/mbs/devicechanged', [this]);
		},

		/**
		 * <p>Widget lifecycle function.</p>
		 */
		postMixInProperties : function() {
			// initialize mixed in parameters if they wern't already
			this.webpage = this.webpage || this.DEFAULT_PAGE;
			this.scaleParams = this.scaleParams || {
				'scale' : 'pixelPerfect',
				'screenPPI' : 97
			};

			this._updatePlatform();
		},

		/**
		 * <p>Widget lifecycle function.</p>
		 */
		postCreate : function() {

			this.__log('postCreate', arguments);

			// Avoid having scrollbars displayed before the iframe content is loaded
			// (It interferes with elements supposed to adapt their size to the iframe boundng box)
			this._forceNoScrollBar = true;

			// generate the device UUID
			this._generateUUID();

			//set up the spinner
			dojo.attr(this._spinner, {
				'src' : dojo.moduleUrl("ibm_mobile", "images/ac16_processing_anim.gif")
			});

			//add in translated strings
			var nls = dojo.i18n.getLocalization("ibm_mobile", "mobile");
			dojo.attr(this._rotateBtn, 'label', nls.MobileBrowserSimulator_rotate);
			dojo.attr(this._closeBtn, 'label', nls.MobileBrowserSimulator_remove_device);

			// match skins platform to device
			var currentPlatformIDString = String(this.platform.id);
			var currentPlatformNameIDList  = currentPlatformIDString.split('\.');
			var currentPlatformNameString= currentPlatformNameIDList[currentPlatformNameIDList.length -1];
            var skinsStorePerEnvironment = null;
	        // create parent store
	        var skinsStoreData = {
		            "label" : "name",
		            "identifier" : "name",
		            "items" : [ {   "name":'default', "platform":'all' } ]
		    };

			this.skinsStore.fetch( { query: { platform: currentPlatformNameString },
	                   onItem: function(item) {
		               var skinsObject = {};
	    	           skinsObject = {
			             "name" : item.name,
			             "platform" : item.platform
			            };
			           // push to store
			           skinsStoreData.items.push(skinsObject);
	            }
			});

			//subscribe to webpage change
			this._subscriptions.push(dojo.subscribe('/mbs/webpage', dojo.hitch(this, function(data) {
				// Avoid having scrollbars displayed before the iframe content is loaded
				// (It interferes with elements supposed to adapt their size to the iframe boundng box)
				this._forceNoScrollBar = true;
				dojo.attr(this.iFrame, 'src', '');
				this._updateSimulator(dojo.hitch(this, function() {
					this.setSrc(data.webpage);
					if(this._radioDiv.firstChild.SkinsRadioGroup.length >= 1)
					{
						this._radioDiv.firstChild.reset();// set default skin
					}
				  }
				));
			})));

			//need to override onClick to return false so it does not try to submit a form
			this._closeBtn.onClick = function() {
				return false;
			};

			//connect to the onClick event of the close button to close the simulator
			dojo.connect(this._closeBtn, 'onClick', this, function(event) {
				//notify
				this.onClose(this);

				//destroy
				this.destroyRecursive(false);
			});

			//connect rotation button
			dojo.connect(this._rotateBtn, 'onClick', this, this._rotate);

			//connect link button for simple preview
			dojo.connect(this._linkBtn, 'onClick', this, this._simplePreview);

			//listen for scale option changes
			this._subscriptions.push(dojo.subscribe("/mbs/changeScale", this, this._scaleChange));

			this._updateSimulator(dojo.hitch(this, function() {


					//set the first page
					this._refresh();

					//need to set the initial skin based on the intial device
					this._updateSkin();

					this.publishDeviceChange();

					//show the sim
					dojo.style(this._wrapper, {
						visibility : "visible"
					});

					//need to use pub/subscribe because IE does not seem to like to connect to the onload event
					this._subscriptions.push(dojo.subscribe("/mbs/iFrame/onload", dojo.hitch(this, function(data) {
						//since using pub/sub need to be sure this pub is for us and not some other frame
						if(data.frame == this.iFrame) {
							this._onIFrameLoad();
						}
					})));

					//create warning tooltip
					new dijit.Tooltip({
						'connectId' : [this._warning],
						'label' : nls.MobileBrowserSimulator_xdomain_warning + "<br />" + nls.xdomain_reduced_func
					});

					//need to reset max height of drop down on a window resize
					dojo.connect(window, "onresize", dojo.hitch(this, function() {
						var btnPos = dojo.position(this._deviceBtn.domNode);
						this._deviceBtn.set('maxHeight', dojo.window.getBox().h - btnPos.y - btnPos.h - 15);
					}));
				}));
		},

		/**
		 * <p>Widget lifecycle function.</p>
		 */
		uninitialize : function () {
			this.__log('uninitialize', arguments);

			//stop the _watchBodySizeTimer
			if (this._watchBodySizeTimer != null)
				clearInterval(this._watchBodySizeTimer);

			//unsubscribe from subscriptions
			dojo.forEach(this._subscriptions, function(subscription) {
				dojo.unsubscribe(subscription);
			});

			//clean up possible connections
			dojo.forEach(this._fitToWindowConnections, function(connection) {
				dojo.disconnect(connection);
			});
		},

		/**
		 * <p>Updates the skin of the simulator based on the devices platform.</p>
		 */
		_updateSkin : function() {
			this.__log('_updateSkin', arguments);

			dojo.removeClass(this.domNode, this._prevSkinClass);
			var newSkinClass;

			//if a known platform use its skin, else use default
			if(this.KNOWN_PLATFORM_SKINS[this.device.platformID]) {
				newSkinClass = this.KNOWN_PLATFORM_SKINS[this.device.platformID];
			} else {
				newSkinClass = this.DEFAULT_SKIN;
			}

			this._prevSkinClass = newSkinClass;
			dojo.addClass(this.domNode, newSkinClass);
		},

		/**
		 * <p>Performs all necessary actions after the content of the iFrame has been updated.</p>
		 */
		_onIFrameLoad : function() {
			this.__log('_onIFrameLoad', arguments);

			this._runInIFrameContext(dojo.hitch(this, function() {
				//keep track of the currently loaded page
				//this.webpage = this.iFrame.contentWindow.location.href;

				//update page defined body tag dimensions
				this._bodyTagDefinedWidth = dojo.body().style.width;
				this._bodyTagDefinedHeight = dojo.body().style.height;

				//deal with viewport tag
				var layoutWidth = (this.device != null)? this.device.viewportDeviceWidth : this.platform.defaultLayoutWidth[0];
				var viewportWidth = this.device.width[0];
				var viewportScale = 1;
				var metaViewportTags = dojo.query("meta[name='viewport']");
				if(metaViewportTags.length > 0) {
					//always use the last meta viewport tag in case there is more then one
					var metaTag = metaViewportTags[metaViewportTags.length-1];

					var contentAttr = dojo.attr(metaTag, 'content');
					if(contentAttr) {
						var params = this._parseCommaSeperatedParams(contentAttr);

						if(params['width']) {
							if(params.width == "device-width") {
								//if the device has a set viewport device width use that, else use default
								var viewportDeviceWidth = this.device.viewportDeviceWidth || this.DEFAULT_VIEWPORT_DEVICE_WIDTH;

								layoutWidth = viewportDeviceWidth;
								viewportWidth = viewportDeviceWidth;
							} else if(!isNaN(params['width'])) {
								layoutWidth = parseInt(params.width);
								viewportWidth = parseInt(params.width);
							} else {
								console.error("Viewport meta tag specified width is not valide: " + params.width);
							}
						}

						if(params['initial-scale'] && !isNaN(params['initial-scale'])) {
							viewportScale = parseFloat(params['initial-scale']);
						}
					}
				}
				this._setLayoutWidth(layoutWidth);
				this._setViewportWidth(viewportWidth);
				this._viewport.scale = viewportScale;

				//on every click we want to be sure the useragent id is set for this device
				dojo.connect(dojo.body(), 'onclick', this, function(event) {

					/* This only works because it seems the extension code that gets called
					 * gets run in sequence with the pages javascript, at least in the case
					 * of the FF extension.  This assumption may not be correct for extensions
					 * written in other browsers, and is guarded against else where in the pages
					 * javascript.  It is not guarded against here because I have yet to find a way
					 * to pause the onclick event until I am sure the browser ID has been set.
					 */
					this._performUseragentIDDependentAction();
				});
			}));

			this._updateSimulator(dojo.hitch(this, function() {
					if (dojo.attr(this.iFrame, 'src') != '')
						this._forceNoScrollBar = false;
				}
			));

		},

		/**
		 * <p>Update the scaling params.</p>
		 *
		 * <p>Subscribed to '/mbs/changeScale' event</p>
		 *
		 * @param scaleParams object which includes the following attributes
		 * 			'scale' : the type of scaling to do
		 * 			'screenPPI' : the PPI of the monitor this simulator is displayed on
		 */
		_scaleChange : function(scaleParams) {
			this.__log('_scaleChange', arguments);

			this.scaleParams = scaleParams;
			this._updateSimulator();
		},

		_updateSimulator : function(refresh) {
            // set qr code
			function createQRDialog(webpageURL)
			{
			  return function(evt){
				var qrCodeDialog = new ibm_mobile.QRCodeDialog({
					'deviceUrl' : webpageURL

				});
				//qrCodeDialog.placeAt(this._qrcodeButtonDevice, 'before');
				dojo.style(qrCodeDialog, "display", "none");

				var ipArray = new Array();
				if ((window.location.hostname !== "localhost")
				    && (window.location.hostname !== "127.0.0.1"))
				    ipArray.push(window.location.hostname);

				if (ips != null) {
					//parse the IPList
					var tempString = ips;
					while (tempString.indexOf(',') != -1) {
					    var currentIP = tempString.substring(0, tempString.indexOf(','));
						if (currentIP !== window.location.hostname)
						    ipArray.push(currentIP);
						tempString = tempString.substring(tempString.indexOf(',') + 1);
					}
					if (tempString !== window.location.hostname)
						ipArray.push(tempString);
				}
				qrCodeDialog.setIPList(ipArray);

	           qrCodeDialog.autoUpdate(true);
				qrCodeDialog.updateQRCode();
				dijit.popup.open({
					parent : this,
					popup : qrCodeDialog,
					around : this.domNode,
					orient : {
						'BR' : 'TR',
						'BL' : 'TL',
						'TR' : 'BR',
						'TL' : 'BL'
					},
					onExecute : function() {
						dijit.popup.close(qrCodeDialog);
						qrCodeDialog.autoUpdate(false);
					},
					onCancel : function() {
						dijit.popup.close(qrCodeDialog);
						qrCodeDialog.autoUpdate(false);
					},
					onClose : function() {
						qrCodeDialog.autoUpdate(false);
					}
				});
				this.set("onBlur",function toggleBlur(){
					dijit.popup.close(qrCodeDialog);
					qrCodeDialog.autoUpdate(false);
				});
				}
			}
			var webpageURL = this.webpage; //set url for QR code
			if((this.sameDomain == true) && (this.smartPreview == "On")  )
			{
				webpageURL = this._smartPreview(this.webpage);
			}
			this._qrcodeButtonDevice.set("onClick", createQRDialog(webpageURL));

			this.__log('_updateSimulator', arguments);

			this._updateCallsCount++;

			this._calcScrollBarSize();
			this._calcScaledDimensions();
			this._scaleContent();

			//update the size of the
			var width;
			var height;
			if(this.isRotated) {
				dojo.addClass(this.domNode, "rotated");
				height = (this._frameWidth + this._horzScrollBarSize);
				width = (this._frameHeight + this._vertScrollBarSize);
			} else {
				dojo.removeClass(this.domNode, "rotated");
				width = (this._frameWidth + this._vertScrollBarSize);
				height = (this._frameHeight + this._horzScrollBarSize);
			}
			width = Math.round(width);
			height = Math.round(height);

			//create the re-size animation
			var animation = dojo.animateProperty({
				node : this.iFrame,
				properties: {
					'width' : {start: dojo.style(this.iFrame, 'width'), end: width, units: "px"},
					'height' : {start: dojo.style(this.iFrame, 'height'), end: height, units: "px"}
				}
			});

			//this is used to force IE to repaint the rest of the screen as the resize takes place
			dojo.connect(animation, 'onAnimate', dojo.hitch(this, function() {
				var div = dojo.create("div", {
					innerHTML : " "
				}, this.domNode, "last");

				dojo.destroy(div);
			}));
			if ((refresh != null) && (refresh != undefined)) {
				dojo.connect(animation, 'onEnd', dojo.hitch(this, refresh));
			}

			//play the animation
			animation.play();

			// set the rotate icon
			if(this.isRotated) {
				this._rotateBtn.attr("iconClass", "rotateRight");
			} else {
				this._rotateBtn.attr("iconClass", "rotateLeft");
			}

			//set the device name
			this._deviceBtn.set('content', String(this.device.name));

			//set max height for the drop down
			var btnPos = dojo.position(this._deviceBtn.domNode);
			this._deviceBtn.set('maxHeight', dojo.window.getBox().h - btnPos.y - btnPos.h - 15);

			this._updateCallsCount--;
		},

		/**
		 * <p>Determine the size of the scroll bars</p>
		 */
		_calcScrollBarSize : function() {
			this.__log('_calcScrollBarSize', arguments);

			this._runInIFrameContext(dojo.hitch(this, function() {
				/* this is a copy of the Dojo 1.5 dojo.window.getBox() function
				 * In Dojo 1.7 the function was changed to special case mobile pages.
				 * The problem is that special casing broke the functionality I needed.
				 * Thus I have copied the Dojo 1.5 version here because it does
				 * what I want it to on any kind of page.
				 */
				function _getClientBox(){
					// summary:
					//		Returns the dimensions and scroll position of the viewable area of a browser window

					var scrollRoot = (dojo.doc.compatMode == 'BackCompat') ? dojo.body() : dojo.doc.documentElement;

					// get scroll position
					var scroll = dojo._docScroll(); // scrollRoot.scrollTop/Left should work
					return { w: scrollRoot.clientWidth, h: scrollRoot.clientHeight, l: scroll.x, t: scroll.y };
				};

				//deal with scroll bars
				var doc = this.iFrame.contentWindow.document.documentElement;
				if ((this._forceNoScrollBar == true)
					|| ((this._isRotating?  doc.scrollHeight : doc.scrollWidth) <= (this.isRotated?  this._frameHeight : this._frameWidth))) {
					this._horzScrollbarVisible = false;
				} else {
					this._horzScrollbarVisible = true;
				}

				if ((this._forceNoScrollBar == true)
					|| ((this._isRotating?  doc.scrollWidth : doc.scrollHeight) <= (this.isRotated?  this._frameWidth : this._frameHeight))) {
					this._vertScrollbarVisible = false;
				} else {
					this._vertScrollbarVisible = true;
				}

				if (this._horzScrollbarVisible == true) {
					dojo.style(this.iFrame, "overflowX", "scroll");
					//dojo.style(this.iFrame, "overflow-x", "scroll");
				} else {
					dojo.style(this.iFrame, "overflowX", "hidden");
					//dojo.style(this.iFrame, "overflow-x", "hidden");
				}

				if (this._vertScrollbarVisible == true) {
					dojo.style(this.iFrame, "overflowY", "scroll");
					//dojo.style(this.iFrame, "overflow-y", "scroll");
				} else {
					dojo.style(this.iFrame, "overflowY", "hidden");
					//dojo.style(this.iFrame, "overflow-y", "hidden");
				}

				//dojo.style(dojo.query("html")[0], "overflow", "hidden");

				//update box as scrollbars may have appeared/disappeard
				clientBox = _getClientBox();
				var frameWidth = dojo.style(this.iFrame, 'width');
				var frameHeight = dojo.style(this.iFrame, 'height');
				this._vertScrollBarSize = (this._vertScrollbarVisible == false) ? 0 : Math.round(frameWidth - clientBox.w);
				this._horzScrollBarSize = (this._horzScrollbarVisible == false) ? 0 : Math.round(frameHeight - clientBox.h);

				//this is because clientWidth does not return what it should for IE, so the math can go all wrong
				if(this._vertScrollBarSize < 0) {
					this.__log("verticle scroll bar size calculated as less then 0: " + this._vertScrollBarSize);
					this._vertScrollBarSize = this.DEFAULT_SCROLL_BAR_WIDTH;
				}
				if(this._horzScrollBarSize < 0) {
					this.__log("horizontal scroll bar size calculated as less then 0: " + this._horzScrollBarSize);
					this._horzScrollBarSize = this.DEFAULT_SCROLL_BAR_WIDTH;
				}

				if (this._watchBodySizeTimer == null) {
					this._watchBodySizeTimer = setInterval(dojo.hitch(this, function() {
				    		// Prevent auto update in special cases
							if ((this._updateCallsCount > 0)
								|| (this._forceNoScrollBar == true)
								|| (this._isRotating == true)
								|| (this._isChangingDevice == true))
								return;
							// Check if the scrollbars should be hidden
							var frameWidth = dojo.style(this.iFrame, 'width');
							var frameHeight = dojo.style(this.iFrame, 'height');
							if (((frameWidth <= (this.isRotated?  this._frameHeight : this._frameWidth))
									&& (this._vertScrollbarVisible == true))
								|| ((frameHeight <= (this.isRotated?  this._frameWidth : this._frameHeight))
									&& (this._horzScrollbarVisible == true))) {
								this._updateSimulator();
								return;
							}
							// Check if the doc box has changed
							try {
								var doc = this.iFrame ? this.iFrame.contentWindow.document.documentElement : null;
							} catch (exception) {
								//publish that we have a cross domain issue
								dojo.publish("/mbs/webpage/xdomain", [{
									'webpage' : this.iFrame.src
								}]);

								this.sameDomain = false;

								//display warning icon
								dojo.style(this._warning, {
									visibility : "visible"
								});
							}
							if (doc == null)
								return;
							if ((doc.scrollHeight != this._lastHeight)
								|| (doc.scrollWidth != this._lastWidth))  {
								this._lastHeight = doc.scrollHeight;
								this._lastWidth = doc.scrollWidth;
								this._updateSimulator();
							}
						}),
						500);
				}

			}));

			this.__log("_vertScrollBarSize: " + this._vertScrollBarSize);
			this.__log("_horzScrollBarSize: " + this._horzScrollBarSize);
		},

		/**
		 * <p>Applies a CSS transformation on a given style</p>
		 */
		_setTransformStyle : function(element, origin, transform, width, height) {
			domStyle.set(element,"transformOrigin", origin );

			// Special case for Firefox as we get an exception stating "InternalError too much recursion"
			if (typeof element.style.MozTransform !== "undefined")
				element.style.MozTransform = transform;
			else
				domStyle.set(element,"transform", transform );

			domStyle.set(element,"width", width );
			domStyle.set(element,"height", height );
		},

		/**
		 * <p>Scales the content of the iframe</p>
		 */
		_scaleContent : function() {
			this.__log('_scaleContent', arguments);

			if (dojo.isIE < 9) {
				// Transform CSS3 declarations are supported in IE7 and in IE8 through the
				// dojo style extension, via the use of a filter matrix.
				// This extension has not been thought to be run in an iFrame context,
				// do not scale content if IE version < 9
				return;
			}

			this._runInIFrameContext(dojo.hitch(this, function() {

				/* if existing width not defined then set to size of device so scale will work
				 * else if existing width defined deal with scaling existing width correctly
				 */
				var newWidth;
				var newHeight;
				if(!this._bodyTagDefinedWidth || this._bodyTagDefinedWidth == "") {
					if(this.isRotated) {
						newWidth = parseInt(this._layout.height);
					} else {
						newWidth = parseInt(this._layout.width);
					}
				} else {
					/* if existing width is absolute don't do any scaling to it
					 * else existing width is relative so scale it correctly
					 */
					if(this._bodyTagDefinedWidth.charAt(this._bodyTagDefinedWidth.length-1) != '%') {
						if(this.isRotated) {
							newWidth = this._bodyTagDefinedHeight;
						} else {
							newWidth = this._bodyTagDefinedWidth;
						}
					} else {
						var percent = parseFloat(this._bodyTagDefinedWidth) / 100;
						if(this.isRotated) {
							newWidth = parseInt(this._layout.height) * percent;
						} else {
							newWidth = parseInt(this._layout.width) * percent;
						}
					}
				}
				/* if existing height not defined then set to size of device so scale will work
				 * else if existing height defined deal with scaling existing width correctly
				 */
				if(!this._bodyTagDefinedHeight || this._bodyTagDefinedHeight == "") {
					if(this.isRotated) {
						newHeight = parseInt(this._layout.width);
					} else {
						newHeight = parseInt(this._layout.height);
					}
				} else {
					/* if existing height is absolute don't do any scaling to it
					 * else existing height is relative so scale it correctly
					 */
					if(this._bodyTagDefinedHeight.charAt(this._bodyTagDefinedHeight.length-1) != '%') {
						if(this.isRotated) {
							newHeight = this._bodyTagDefinedWidth;
						} else {
							newHeight = this._bodyTagDefinedHeight;
						}
					} else {
						var percent = parseFloat(this._bodyTagDefinedHeight) / 100;
						if(this.isRotated) {
							newHeight = parseInt(this._layout.width) * percent;
						} else {
							newHeight = parseInt(this._layout.height) * percent;
						}
					}
				}
				//calculate scale ratio
				var ratio = 1;
				switch(this.scaleParams.scale) {
					case 'pixelPerfect': {
						ratio = this._viewport.scale;
						break;
					}
					case 'deviceSize':
					case 'fitToWindow': {
						ratio = (this._frameWidth / this._layout.width) * this._viewport.scale;
						break;
					}
				}
				var scaleNode = dojo.body();
				if (typeof scaleNode !== 'undefined') {
					if (ratio == 1) {
						// if the transformation is an identity, then we don't set it because we have already observed
						// UI elements being moved only by setting an identity transformation which normally
						// should have no effect
						//this._setTransformStyle(scaleNode,"0 0", "none", newWidth + "px", newHeight + "px" );
						this._setTransformStyle(scaleNode,"", "", "", "" );
					} else {
						//when transforming we need to set the body size so relatively sized elements will be correct
						this._setTransformStyle(scaleNode,"0 0", "scale(" + ratio + ")", newWidth + "px", newHeight + "px" );
					}
				}
			}));
		},

		_deviceChange : function(device) {
			this.__log('_deviceChange', arguments);

			// Avoid having scrollbars displayed before the iframe content is loaded
			// (It interferes with elements supposed to adapt their size to the iframe boundng box)
			this._forceNoScrollBar = true;
			this._isChangingDevice = true;

			this._isRotating = false;
			this.isRotated = false;

			// Set device and platform
			this.device = device;
			this._updatePlatform();

			// generate the device UUID
			this._generateUUID();

			//update everything
			this._updateSkin();

			//reset source as user agent may have changed
			dojo.attr(this.iFrame, 'src', '');
			this._updateSimulator(dojo.hitch(this, function() {
					this._refresh();
				}
			));

			//this._deviceBtn.closeDropDown();

			this._isChangingDevice = false;

			// notify
			this.onDeviceChange(device);
		},

		_calcScaledDimensions : function() {
			this.__log('_calcScaledDimensions', arguments);

			//disconnect from events for fit to window
			dojo.forEach(this._fitToWindowConnections, function(connection) {
				dojo.disconnect(connection);
			});
			this._fitToWindowConnections = [];

			switch(this.scaleParams.scale) {
				case 'pixelPerfect': {
					this._frameWidth = parseInt(this._layout.width);
					this._frameHeight = parseInt(this._layout.height);
					break;
				}
				case 'deviceSize': {
					this._frameWidth = this._devicePixelsToCSSPixels(this.device.width);
					this._frameHeight = this._devicePixelsToCSSPixels(this.device.height);

					break;
				}
				case  'fitToWindow': {
					//resize on set
					this._fitToWindow();

					//add listeners for events that we then cause the sim to need to be resized
					var reactToChange = function() {
						this._fitToWindow();
						this._updateSimulator();
					};
					this._fitToWindowConnections.push(dojo.connect(window, 'onresize', this, reactToChange));
					this._fitToWindowConnections.push(dojo.connect(this, 'onRotate', this, reactToChange));

					break;
				}
			}
		},

		/**
		 * Update the source used by the iframe
		 *
		 * @param webpage new source for the iframe
		 */
		setSrc : function(/* String */ webpage) {
			this.__log('setSrc', arguments);

			//if blank src then use the default page
			this.webpage = webpage || this.DEFAULT_PAGE;

			//hide the warning, it will reappear for the new page if need be
			dojo.style(this._warning, {
				visibility : "hidden"
			});

			//hide the iframe and show spinner
			dojo.style(this.iFrame, {
				visibility : "hidden"
			});
			dojo.style(this._spinner, {
				visibility : "visible"
			});

			//setting the new source is a user agent dependent action
			this._performUseragentIDDependentAction(dojo.hitch(this, function() {
				this.__log("setSrc: Call back from _performUseragentIDDependentAction", arguments);
				var deferred = new dojo.Deferred();

				var sub = dojo.subscribe("/mbs/iFrame/onload", dojo.hitch(this, function(data) {
					//since using pub/sub need to be sure this pub is for us and not some other frame
					if(data.frame == this.iFrame) {
						dojo.unsubscribe(sub);

						//hide the spinner and show the iframe
						dojo.style(this._spinner, {
							visibility : "hidden"
						});
						dojo.style(this.iFrame, {
							visibility : "visible"
						});

						deferred.callback(true);
					}
				}));

				//make every request unique to prevent caching
				var uuid = (new Date()).valueOf();
				var indexOfSplit = this.webpage.indexOf('?');
				var uri = null;
				var queryStr = null;

				//split up webpage into uri and query strings
				if(indexOfSplit != -1) {
					uri = this.webpage.substring(0, indexOfSplit);
					queryStr = this.webpage.substring(indexOfSplit+1, this.webpage.length);
				} else {
					uri = this.webpage;
				}

				/* if existing query edit or add to existing prevent cache field
				 * else add new query string with prevent cache field */
				if(queryStr) {
		console.error("queryStr: " + queryStr);

					var query = dojo.queryToObject(queryStr);
					query['mbs.preventCache'] = uuid;
					queryStr = dojo.objectToQuery(query);
				} else {
					queryStr = "mbs.preventCache=" + uuid;
				}
				var urlToLoad = uri + "?" + queryStr;
				//Smart Preview: make sure url device type matches the device platform if the url is on the same host and smart preview is on
				if( (this.sameDomain == true) && (this.smartPreview == "On") ) //if from different domain, display arbitrary page
				{
					urlToLoad = this._smartPreview(urlToLoad);
				}
				//set new source
				dojo.attr(this.iFrame, 'src', urlToLoad);

				return deferred;
			}));
		},

		/**
		 * <p>Refreshes the iframe with the currently loaded page</p>
		 */
		_refresh : function() {
			this.__log('_refresh', arguments);
			this.setSrc(this.webpage);
		},

		/**
		 * <p>Sets the devices url to match the device platform type </p>
		 */
		_smartPreview : function(urlToLoad) {
			var hostMain = 'localhost';
			var portMain = '10081';
			var androidUrl = '/android/assets/www/index.html';
			var iosUrl = '/ios/www/index.html';
			var windowsUrl = '/windows/www/index.html';

			var url = 'http://' + hostMain + ':' + portMain;
			var currentPlatform = String(this.platform.id);

			if (currentPlatform.indexOf('android') !== -1)
				return url + androidUrl;
			else if (currentPlatform.indexOf('ios') !== -1 ||
							 currentPlatform.indexOf('iphone') !== -1 ||
						   currentPlatform.indexOf('ipad') !== -1)
				return url + iosUrl;
		  else if (currentPlatform.indexOf('windows') !== -1)
				return url + windowsUrl;
			return;
		},

		/**
		 * <p>Rotates the simulator</p>
		 */
		_rotate : function() {
			this.__log('_rotate', arguments);
			if (this._isRotating == true)
				return;
			this._isRotating = true;

			this.isRotated = !this.isRotated;

			// if supports transforms animate a rotation
			// else just swap width & height
			if(dojo.isWebKit || dojo.isMozilla) {
				var transform;
				dojo.forEach(
					['transform', 'WebkitTransform', 'msTransform',
					'MozTransform', 'OTransform'],
					function (name) {
						if (typeof dojo.body().style[name] != 'undefined') {
						transform = name;
					}
				});

				var angles;
				if(this.isRotated) {
					angles = [0, -1];
				} else {
					angles = [0, 1];
				}

				var initialWidth = dojo.style(this._parent, "width");
				var initialHeight = dojo.style(this._parent, "height");

				var rotation = new dojo.Animation({
					duration: 1000,
				    curve: angles,
				    onAnimate: dojo.hitch(this, function (v) {
				        this._wrapper.style[transform] = 'rotate(' + (v*90) + 'deg)';

				        var tbopacity = 1-2*(v<0? -v : v);
				        if (tbopacity <0)
				        	tbopacity = 0;
				        dojo.style(this._devicetoolbar, {
							opacity: tbopacity
						});

				    }),
				    onEnd: dojo.hitch(this, function(){
						//swap height and width
						dojo.style(this.iFrame, {
							height : dojo.style(this.iFrame, "width") + "px",
							width : dojo.style(this.iFrame, "height") + "px"
						});

						//reset rotation
						dojo.style(this._wrapper, {
							'MozTransform' : "rotate(0deg)",
							'WebkitTransform' : "rotate(0deg)"
						});

						// First, we reset properly the attributes of the body tag of the iframe content
						this._scaleContent();
						// Then, we update the simulator for possible scrollbar visibility change
						this._updateSimulator();

						this._isRotating = false;

						dojo.style(this._devicetoolbar, {
							opacity: 1
						});

						//send out notification
						this.onRotate();


					})
				}).play();
			} else {
				//for IE just swap width & height because it can't animate
				dojo.style(this.iFrame, {
					height : dojo.style(this.iFrame, "width") + "px",
					width : dojo.style(this.iFrame, "height") + "px"
				});

				//update the sim
				this._updateSimulator();

				this._isRotating = false;
				//send out notification
				this.onRotate();
			}
		},


		/**
		 * <p>Open a simple preview in a new tab</p>
		 */
		_simplePreview : function() {
			  var webpageURL = this.webpage;
			  if((this.sameDomain == true) && (this.smartPreview == "On")  )
			  {
				webpageURL = this._smartPreview(this.webpage);
			  }
			  var win=window.open(webpageURL, '_blank');
			  win.focus();
		},


		/**
		 * <p>Sets the scaling of the simulator such that it fits in the browser window area.</p>
		 */
		_fitToWindow : function() {
			this.__log('_fitToWindow', arguments);

			//compensate for border around iframe
			var simMarginBox = dojo.marginBox(this.domNode);
			var iframeMarginBox = dojo.marginBox(this.iFrame);
			var containerBox = dojo.marginBox(dojo.byId("_centerPane")); //<-- this isn't great because it refers to node in containing page
			var maxHeight = containerBox.h - (simMarginBox.h - iframeMarginBox.h) - 10;

			//figure out scale ratio dependent on rotation
			var ratio;
			if(this.isRotated) {
				ratio = maxHeight / this._layout.width;
			} else {
				ratio = maxHeight / this._layout.height;
			}

			//never have ratio greater then 1
			if(ratio > 1) {
				ratio = 1;
			}

			//update frame dimensions
			this._frameWidth = this._layout.width * ratio;
			this._frameHeight = this._layout.height *  ratio;
		},

		/**
		 * <p>Runs the given function with the context of dojo set to that of the iFrame.</p>
		 *
		 * <p>If the iframe has no document loaded then the function is not run.</p>
		 *
		 * @param func run this function in the context of the iFrame
		 */
		_runInIFrameContext : function(func) {
			//can only run iframe context if there is a content window and document loaded

			try {
				// Strange IE8 behavior... Evaluating this.iFrame.contentWindow throws "Unspecified error."
				// if iFrane is not loaded.
				this.iFrame.contentWindow;
			} catch (e) {
				return;
			}

			if(this.iFrame.contentWindow) {
				var contentWindowDoc;
				try {
					contentWindowDoc = this.iFrame.contentWindow.document;
				} catch(exception) {
					//publish that we have a cross domain issue
					dojo.publish("/mbs/webpage/xdomain", [{
						'webpage' : this.iFrame.src
					}]);

					this.sameDomain = false;

					//display warning icon
					dojo.style(this._warning, {
						visibility : "visible"
					});
				}
				if(contentWindowDoc) {
					var oldWin = dojo.global;
					var oldDoc = dojo.doc;
					//run the supplied function in the context of the iFrame
					try {
						dojo.setContext(this.iFrame.contentWindow, this.iFrame.contentWindow.document);
						func();
					} catch (exception) {
						this.__log(exception);
					}
					finally {
						dojo.setContext(oldWin, oldDoc);
					}
				}
			}
		},

		_setLayoutWidth : function(/* int */layoutWidth) {
			this.__log('_setLayoutWidth', arguments);

			var ratio = this.device.height / this.device.width;
			this._layout.width = layoutWidth;
			this._layout.height = layoutWidth * ratio;
		},

		_setViewportWidth : function(/* int */viewportWidth) {
			this.__log('_setViewportWidth', arguments);

			var ratio = this.device.height / this.device.width;
			this._viewport.width = viewportWidth;
			this._viewport.height = viewportWidth * ratio;
		},

		/**
		 * <p>Generate the device UUID</p>
		 */
		_generateUUID : function() {
			var S4 = function() {
				return (((1+Math.random())*0x10000)|0).toString(16).substring(1);
			};

			this.uuid = (S4()+"-"+S4()+"-"+S4()+"-"+S4());
		},

		/**
		 * <p>Update the platform</p>
		 */
		_updatePlatform : function() {
			this.__log('_updatePlatform', arguments);

			//set up the default layout
			this.deviceStore.fetchItemByIdentity({
				'identity' : this.device.platformID,
				'onItem' : function(platform) {
					this.platform = platform;
					this._setLayoutWidth((this.device != null)? this.device.viewportDeviceWidth : this.platform.defaultLayoutWidth[0]);
				},
				'onError' : function(error) {
					console.error("Could not find the matching platform item for this device: " + error);
				},
				'scope' : this
			});
		},

		/**
		 * <p>Converts device pixels into CSS pixels.</p>
		 *
		 * <p>For example, if the device screen is 320 device pixels wide,
		 * this funciton will return the number CSS pixels to use to display
		 * that device width on the users monitor so it renders the same physical
		 * size as the device.</p.
		 *
		 * @param devicePixels number of devices pixels to turn into a number of
		 * monitor pixels
		 * @returns {Number} the number of CSS pixels to use to display the given
		 * number of <code>devicePixels</code> on the users monitor
		 */
		_devicePixelsToCSSPixels : function(devicePixels) {
			var devicePPI = parseInt(this.device.ppi);
			var monitorInches = devicePixels / devicePPI;
			return monitorInches * this.scaleParams.screenPPI;
		},

		/**
		 * <p>Parses a string in the format "param1=value1, param2=value2, ..." and returns
		 * an object.</p>
		 *
		 * @param toParse
		 */
		_parseCommaSeperatedParams : function(/* String */toParse) {
			var params = {};
			dojo.forEach(toParse.split(","), function(param) {
				var splitParam = dojo.trim(param).split("=");
				params[splitParam[0]] = splitParam[1];
			});

			return params;
		},

		/**
		 * <p>Publishes a request for a specific useragent id to be used</p>
		 */
		_performUseragentIDDependentAction : function(action) {
			this.__log('_performUseragentIDDependentAction', arguments);

			dojo.publish('/mbs/useragent', [{
				'useragent' : this.device.agentID,
				'action' : action
			}]);
		},

		/**
		 * <p>Used to log debugging messages</p>
		 *
		 * @param msg
		 * @param args
		 */
		__log : function(msg, args) {
			if(this.__DEBUG_ENABLED) {
				msg = "MobileBrowserSimulator: " + msg;
				console.log(msg);
				if(args && args.length > 0) {
					console.log(args);
				}
			}
		}
	}); //end dojo.declair

	return MobileBrowserSimulator;
});

