/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Mobile Browser Simulator
 * (c) Copyright IBM Corporation 2011,2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

/**
 * Widget designed to contain multiple MobileBrowserSimulator widgets
 */
define("ibm_mobile/MobileBrowserSimulatorContainer",[
	"dojo",
	"dijit",
	"dijit/_Widget",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dojo/text!ibm_mobile/templates/MobileBrowserSimulatorContainer.html",
	"ibm_mobile/MobileBrowserSimulator"
], function(dojo, dijit, _Widget, _TemplatedMixin, _WidgetsInTemplateMixin, mobileBrowserSimulatorContainerTemplate) {

	var MobileBrowserSimulatorContainer = dojo.declare("ibm_mobile.MobileBrowserSimulatorContainer", [dijit._Widget, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin ], {

    platforms : null,

		/** Used to debug this widget */
		__DEBUG_ENABLED : false,

		// PUBLIC PARAMS
		/** @type dojo.data.ItemFileReadStore */
		deviceStore : null,

		/** @type dojo.data.ItemFileReadStore */
		skinsStore : null,

		/** @type dojo.data.ItemFileReadStore */
	    environmentsStore : null,

		/** need to keep track of current webpage for when creating new sims */
		webpage : null,
		/** the default platform if set */
		defaultPlatform : null,

		/** smart preview boolean */
		smartPreview : null,

		iosVersion : null,

		iosVersionOn : null,

		/** showSingleEnvironment var */
		showSingleEnvironment : null,

		// WIDGET PARAMS
		templateString : mobileBrowserSimulatorContainerTemplate,

		// PRIVATE CONSTANTS
		DEFAULT_DEVICE_NAME_IPHONE : "Apple iPhone 6s",
		DEFAULT_DEVICE_NAME_BlACKBERRY : "Blackberry Bold 9930",
		DEFAULT_DEVICE_NAME_IPAD : "Apple iPad4",
		DEFAULT_DEVICE_NAME_WINDOWS8 : "ASUS VivoTab Smart",
		DEFAULT_DEVICE_NAME_ANDROID : "Google Nexus 5",
		DEFAULT_DEVICE_NAME_WINDOWSPHONE8 : "HTC Radar 4G",

		DEFAULT_DEVICE_NAMES : {
			'platform.blackberry' : 'Blackberry Bold 9930',
			'platform.android' : 'Google Nexus 5',
			'platform.ios.ipad' : 'Apple iPad4',
			'platform.ios.iphone' : 'Apple iPhone 6s',
			'platform.iphone' : 'Apple iPhone 6s',
			'platform.windowsphone' : 'HTC Radar 4G',
			'platform.windows8' : 'ASUS VivoTab Smart'
		},

		// PRIVATE
		/** keeps track of the sims instances */
		sims : null,

		/** keeps track of the sims TD tags added to this container so they can be removed on close */
		_simsTD : null,

		/** need to keep track of these for when creating new sims */
		_currScaleParams : null,

		/** need to keep track of current webpage for when creating new sims */
		webpage : null,

		/** keep track of subscriptions for destruction purposes */
		_subscriptions : null,

		// DOJO ATTATCH POINTS
		_deviceRow : null,

		/**
		 * <p>Widget lifecycle function.</p>
		 *
		 * @param params
		 */
		constructor : function(params) {
			//init fields
			this.sims = new Array();
			this._simsTD = {};
			this._subscriptions = [];
			this._currScaleParams = {
					'scale' : 'pixelPerfect'
			};
		},

		/**
		 * <p>Widget lifecycle function.</p>
		 */
		postMixInProperties : function() {
			//listen for scale option changes
			this._subscriptions.push(dojo.subscribe("/mbs/changeScale", this, function(scaleParams) {
				this._currScaleParams = scaleParams;
			}));

			//subscribe to webpage changes
			this._subscriptions.push(dojo.subscribe('/mbs/webpage', dojo.hitch(this, function(data) {
				this.webpage = data.webpage;
			})));
		},

		/**
		 * <p>Widget lifecycle function.</p>
		 */
		postCreate : function() {
			var deviceFound = false;
      if (this.smartPreview == "On" && this.platforms !== null) {
          var platformList = (this.platforms).split(',');
          for (var i = 0; i < platformList.length; i++) {
              var platformID = "platform." + platformList[i];
              this.deviceStore.fetchItemByIdentity({
                  'identity' : this.DEFAULT_DEVICE_NAMES[platformID],
                  'onItem' : function(device) {
                      // if found default device then load it
                      //  else load first device in store
                      if(device) {
                          deviceFound = true;
                          this.addDevice(device);
                      } else {
                          deviceFound = true;
                          this._addFirstDevice();
                      }
                  },
                  'onError' : function(error) {
                  console.log("ON ERROR 1: "+ error);
                      deviceFound = true;
                      //try loading the first device in the store
                      this._addFirstDevice();
                  },
                  'scope' : this
              });
          }
      }
      if (deviceFound == true)
          return;


			if ( (this.defaultPlatform != null) && (this.smartPreview == "Off")) { //only one environment case
			    var platformID = "platform." + this.defaultPlatform;
					this.deviceStore.fetchItemByIdentity({
							'identity' : this.DEFAULT_DEVICE_NAMES[platformID],
							'onItem' : function(device) {
									// if found default device then load it
									//  else load first device in store
									if(device) {
											deviceFound = true;
											this.addDevice(device);
									} else {
											deviceFound = true;
											this._addFirstDevice();
									}
							},
							'onError' : function(error) {
							console.log("ON ERROR 1: "+ error);
									deviceFound = true;
									//try loading the first device in the store
									this._addFirstDevice();
							},
							'scope' : this
					});
			 }
			if (deviceFound == true)
			    return;

		    //if smart preview case, loop device store fetch from list created from index.js containing the environments
			if( (this.smartPreview == "On") && (this.environmentsStore != null) )
			{
			  this.environmentsStore.fetch({
				    'start': 0,
				    'onItem' : function(environment) {
						var platformID =  "platform." + environment.name;
						if(platformID == "platform.iphone")
						{
							platformID = "platform." + "ios.iphone";
						}
						if(platformID == "platform.ipad")
						{
							platformID = "platform." + "ios.ipad";
						}
						if(platformID == "platform.windowsphone8")
						{
							platformID = "platform." + "windowsphone";
						}
						if(platformID != "platform.blackberry") //non blackberry 10 device should not be added to smart preview due to cordova support
						{
							if(platformID == "platform.blackberry10")
							{
								platformID = "platform." + "blackberry"; //set platformID for blacberry10 to blackberry only to fetch device
							}

						    if(platformID == "platform.iphone" || platformID == "platform.ios.iphone"){

						    	 this.deviceStore.fetchItemByIdentity({
						    		'identity' : this.DEFAULT_DEVICE_NAME_IPHONE,
						    		'onItem' : function(device) {
						    			// if found default device then load it
						    			//  else load first device in store

						    			if(device) {
						    				deviceFound = true;
						    				this.addDevice(device);
						    			} else {
						    				deviceFound = true;
						    				this._addFirstDevice();
						    			}
						    		},
						    		'onError' : function(error) {
						    		console.log('onError 3 : '+ error);
						    			//try loading the first device in the store
						    			deviceFound = true;
						    			this._addFirstDevice();
						    		},
						    		'scope' : this
						    	});
						    }
						    else if (platformID == "platform.android"){
						    	this.deviceStore.fetchItemByIdentity({
						    		'identity' : this.DEFAULT_DEVICE_NAME_ANDROID,
						    		'onItem' : function(device) {
						    			// if found default device then load it
						    			//  else load first device in store
						    			if(device) {
					    				    deviceFound = true;

						    				this.addDevice(device);
						    			} else {
						    				deviceFound = true;
						    				this._addFirstDevice();
						    			}
						    		},
						    		'onError' : function(error) {
						    		console.log("ON ERROR 1: "+ error);
						    			deviceFound = true;
						    			//try loading the first device in the store
						    			this._addFirstDevice();
						    		},
						    		'scope' : this
						    	});

						    }
							else{

								this.deviceStore.fetch({  //find the default device and add it to mbs
									'query': {
										'type': 'device',
										'platformID' : platformID
									},
									'start': 0,
									'count': 1,
									'onItem' : function(device) {
										if(device) {
											this.addDevice(device);
											deviceFound = true;
										} else {
											console.error("No device for platform " + platformID + ".");
										}
									},
									'onError' : function(error) {
										console.error("Error while trying to load a device for platform " + platformID + ": " + error);
									},
									'scope': this
								});
							}

					    }
				    },
				    'onError' : function(error) {
					    console.error("Error while trying to load environment from environment list object. "  + error);
				    },
				    'scope': this
			    });
			}

			if (deviceFound == true)
			    return;

			this.deviceStore.fetchItemByIdentity({
				'identity' : this.DEFAULT_DEVICE_NAME_IPHONE,
				'onItem' : function(device) {
					// if found default device then load it
					//  else load first device in store
					if(device) {
						this.addDevice(device);
					} else {
						this._addFirstDevice();
					}
				},
				'onError' : function(error) {
					//try loading the first device in the store
					this._addFirstDevice();
				},
				'scope' : this
			});
		},

		/**
		 * <p>Widget lifecycle function.</p>
		 */
		uninitialize : function () {
			//unsubscribe from subscriptions
			dojo.forEach(this._subscriptions, function(subscription) {
				dojo.unsubscribe(subscription);
			});
		},

		/**
		 * <p>Adds a device to the simulator.</p>
		 *
		 * @param device Device to add to the simulator
		 */
		addDevice : function(device) {
			//create the simulator
			var sim = new ibm_mobile.MobileBrowserSimulator({
				'device' : device,
				'deviceStore' : this.deviceStore,
				'skinsStore' : this.skinsStore,
				'environmentsStore' : this.environmentsStore,
				'smartPreview' : this.smartPreview,
				'iosVersion' : this.iosVersion,
				'iosVersionOn' : this.iosVersionOn,
				'scaleParams' : this._currScaleParams,
				'webpage'     : this.webpage,
				'__DEBUG_ENABLED' : this.__DEBUG_ENABLED
			});

			//wire up simulator close
			dojo.connect(sim, 'onClose', this, this._deviceRemoved);

			//create room in table and add simulator
			var td = dojo.create("td", {}, this._deviceRow, "last");
			sim.placeAt(td, 'only');
			this._simsTD[sim] = td;
			this.sims[this.sims.length] = sim;
		},

		/**
		 * Deal with removing a simulator from the table
		 *
		 * @param sim simulator to remove
		 */
		_deviceRemoved : function(sim) {
			for (var i = 0 ; i < this.sims.length ; i++) {
				if (this.sims[i] = sim) {
					this.sims.splice(i, 1);
					break;
				}
			}
			dojo.destroy(this._simsTD[sim]);
			delete this._simsTD[sim];
		},

		/**
		 * Adds the first device from the store to the simulator
		 */
		_addFirstDevice : function() {
			this.deviceStore.fetch({
				'query': {
					'type': 'device'
				},
				'start': 0,
				'count': 1,
				'onItem' : function(device) {
					if(device) {
						this.addDevice(device);
					} else {
						console.error("Device list is empty so could not load first device.");
					}
				},
				'onError' : function(error) {
					console.error("Error while trying to load the first device: " + error);
				},
				'scope': this
			});
		}
	}); //end dojo.declair

	return MobileBrowserSimulatorContainer;
});
