/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Mobile Browser Simulator
 * © Copyright IBM Corporation 2011. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

define("ibm_mobile/QRCodeWidget", [ "dojo", "dojo/dom-construct", "dojo/html",
		"dijit", "dijit/_Widget", "ibm_mobile/QRCode" ], function(dojo,
		domConstruct, html, dijit, _Widget, qrcode) {

	var QRWidget = dojo.declare("ibm_mobile.QRCodeWidget", [ dijit._Widget ], {
		text : null,
		cellSize : 4,
		margin : 8,
		rendering : "image",
		typeNumber : 9, // 1 to 10
		errorCorrectLevel : 'L', // L','M','Q','H'

		_setTypeNumberAttr : function(v) {
			var value = v;
			value = Math.round(value);
			if(value < 1)
				value = 1;
			if(value > 10)
				value = 10;
			this.typeNumber = value;
			this._render();
		},

		_setCellSizeAttr : function(value) {
			this.cellSize = value;
			this._render();
		},

		_setTextAttr : function(value) {
			this.text = value;
			this._render();
		},

		_setRenderingAttr : function(value) {
			this.rendering = value;
			this._render();
		},

		_setErrorCorrectLevelAttr : function(value) {
			this.errorCorrectLevel = value;
			this._render();
		},
		
		_setMarginAttr : function(value) {
			this.margin = value;
			this._render();
		},

		_render : function() {
			if (this.text == null)
				return;
			var qr = new qrcode().qrcode()(this.typeNumber, this.errorCorrectLevel);
			qr.addData(this.text);
			qr.make();
			var r = this.rendering;
			var tag;

			switch (r) {
			case "table":
				tag = qr.createTableTag(this.cellSize, this.margin);
				break;
			default:
			case "image":
				tag = qr.createImgTag(this.cellSize, this.margin);
				break;
			}

			var node = this.domNode;
			html.set(node, tag);
		},

		buildRendering : function() {
			this.inherited(arguments);
			this._render();
		},

		resize : function(b) {
			this.inherited(arguments);
		}
	});
	return QRWidget;
});
